(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
    typeof define === 'function' && define.amd ? define('dynamic-typed-array', factory) :
    (global.DynamicTypedArray = factory());
}(this, function () { 'use strict';

    var babelHelpers = {};

    babelHelpers.classCallCheck = function (instance, Constructor) {
      if (!(instance instanceof Constructor)) {
        throw new TypeError("Cannot call a class as a function");
      }
    };

    babelHelpers;

    // Tiny string-based Symbol polyfill
    /* istanbul ignore next */
    if (typeof Symbol !== 'function') {
        var _symbol = function _Symbol() {
            return Math.random().toString(16).slice(2);
        };
        _symbol.for = function (key) {
            return key;
        };
        _symbol.iterator = '@@iterator';
        var global = Function('return this')();
        Object.defineProperty(global, 'Symbol', { value: _symbol });
    }

    // The default growth policy grows the buffer by 1.5x the requested capacity.
    var defaultGrowthPolicy = function defaultGrowthPolicy(minCapacity) {
        return 3 * minCapacity >>> 1;
    };
    // The default allocator simply creates a new array buffer.
    var defaultAllocator = function defaultAllocator(byteLength) {
        return new ArrayBuffer(byteLength);
    };
    // The default dealloctor does nothing.
    var defaultDeallocator = function defaultDeallocator(buffer) {};
    // The default reallocator uses the ArrayBuffer::transfer function, if it is available.
    // Otherwise it simply allocates a new buffer and copies the old data over.
    var defaultReallocator = ArrayBuffer['transfer'] || function defaultReallocator(oldBuffer, newByteLength, allocator, deallocator) {
        var newBuffer = allocator(newByteLength);
        // Copy data over to the new buffer
        var oldView = new Uint8Array(oldBuffer);
        var newView = new Uint8Array(newBuffer);
        if (newView.length <= oldView.length) {
            oldView = oldView.subarray(0, newView.length);
        }
        newView.set(oldView);
        deallocator(oldBuffer);
        return newBuffer;
    };

    var ArrayIterator = function () {
        function ArrayIterator(array, start, end) {
            babelHelpers.classCallCheck(this, ArrayIterator);

            this._array = array;
            this._start = start;
            this._end = end;
            this._result = { value: array[start], done: false };
        }

        ArrayIterator.prototype.next = function next() {
            if (this._start < this._end) {
                this._result.value = this._array[this._start++];
                this._result.done = false;
            } else {
                this._result.value = undefined;
                this._result.done = true;
            }
            return this._result;
        };

        return ArrayIterator;
    }();
    function isTypedArrayConstructor(obj) {
        switch (obj) {
            case Int8Array:
            case Uint8Array:
            case Uint8ClampedArray:
            case Int16Array:
            case Uint16Array:
            case Int32Array:
            case Uint32Array:
            case Float32Array:
            case Float64Array:
                return true;
        }
        return false;
    }
    function isNumber(obj) {
        return typeof obj === 'number';
    }
    function isFunction(obj) {
        return typeof obj === 'function';
    }
    function isArrayLike(obj) {
        return typeof obj !== 'function' && typeof obj.length === 'number';
    }
    function isIterable(obj) {
        return Symbol.iterator in obj && typeof obj[Symbol.iterator] === 'function';
    }

    var DynamicTypedArray = function () {
        function DynamicTypedArray() {
            var constructor = arguments.length <= 0 || arguments[0] === undefined ? Float64Array : arguments[0];
            var init = arguments.length <= 1 || arguments[1] === undefined ? 0 : arguments[1];

            var _ref = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];

            var _ref$growthPolicy = _ref.growthPolicy;
            var growthPolicy = _ref$growthPolicy === undefined ? defaultGrowthPolicy : _ref$growthPolicy;
            var _ref$allocator = _ref.allocator;
            var allocator = _ref$allocator === undefined ? defaultAllocator : _ref$allocator;
            var _ref$deallocator = _ref.deallocator;
            var deallocator = _ref$deallocator === undefined ? defaultDeallocator : _ref$deallocator;
            var _ref$reallocator = _ref.reallocator;
            var reallocator = _ref$reallocator === undefined ? defaultReallocator : _ref$reallocator;
            babelHelpers.classCallCheck(this, DynamicTypedArray);

            if (!isTypedArrayConstructor(constructor)) {
                throw new TypeError('The constructor must be a typed array constructor function!');
            }
            if (!isFunction(growthPolicy)) {
                throw new TypeError('The growthPolicy must be a function!');
            }
            if (!isFunction(allocator)) {
                throw new TypeError('The allocator must be a function!');
            }
            if (!isFunction(deallocator)) {
                throw new TypeError('The deallocator must be a function!');
            }
            if (!isFunction(reallocator)) {
                throw new TypeError('The reallocator must be a function!');
            }
            this._array = undefined;
            this._length = 0;
            this._constructor = constructor;
            this._growthPolicy = growthPolicy;
            this._allocator = allocator;
            this._deallocator = deallocator;
            this._reallocator = reallocator;
            if (isNumber(init)) {
                if (init < 0) {
                    throw new RangeError('The initial size must not be negative!');
                }
                this._ensureCapacity(init);
                this._length = init;
            } else if (isArrayLike(init)) {
                var length = init.length;

                this._ensureCapacity(length);
                this._length = length;
                for (var i = 0; i < length; ++i) {
                    this._array[i] = init[i];
                }
            } else if (isIterable(init)) {
                var _iteratorNormalCompletion = true;
                var _didIteratorError = false;
                var _iteratorError = undefined;

                try {
                    for (var _iterator = init[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                        var value = _step.value;

                        this.push(value);
                    }
                } catch (err) {
                    _didIteratorError = true;
                    _iteratorError = err;
                } finally {
                    try {
                        if (!_iteratorNormalCompletion && _iterator.return) {
                            _iterator.return();
                        }
                    } finally {
                        if (_didIteratorError) {
                            throw _iteratorError;
                        }
                    }
                }
            } else {
                throw new TypeError('The initilizer must be a number or an iterable object.');
            }
        }

        DynamicTypedArray.prototype.size = function size() {
            return this._length;
        };

        DynamicTypedArray.prototype.capacity = function capacity() {
            return this._array.length;
        };

        DynamicTypedArray.prototype.get = function get(index) {
            if (index < this._length) {
                return this._array[index];
            }
        };

        DynamicTypedArray.prototype.set = function set(index, value) {
            if (index < this._length) {
                this._array[index] = value;
            }
        };

        DynamicTypedArray.prototype.push = function push() {
            var _arguments = arguments;
            var length = _arguments.length;

            this._ensureCapacity(this._length + length);
            for (var i = 0; i < length; ++i) {
                this._array[this._length++] = arguments[i];
            }
        };

        DynamicTypedArray.prototype.pop = function pop() {
            return this._length <= 0 ? undefined : this._array[--this._length];
        };

        DynamicTypedArray.prototype.forEach = function forEach(callback, thisArg) {
            var length = this._length;
            for (var i = 0; i < length; ++i) {
                callback.call(thisArg, this._array[i], i);
            }
        };

        DynamicTypedArray.prototype[Symbol.iterator] = function () {
            return new ArrayIterator(this._array, 0, this._length);
        };

        DynamicTypedArray.prototype.toArray = function toArray() {
            return Array.prototype.slice.call(this._array, 0, this._length);
        };

        DynamicTypedArray.prototype.toJSON = function toJSON() {
            return this.toArray();
        };

        DynamicTypedArray.prototype.toString = function toString() {
            return this.toArray().toString();
        };

        DynamicTypedArray.prototype._ensureCapacity = function _ensureCapacity(minCapacity) {
            if (!this._array || minCapacity > this.capacity()) {
                var newCapacity = this._growthPolicy(minCapacity);
                if ((newCapacity | 0) !== newCapacity) {
                    throw new TypeError('The computed new capacity (' + newCapacity + ') is not an integer value.');
                } else if (newCapacity < minCapacity) {
                    throw new RangeError('The computed new capacity (' + newCapacity + ') is smaller than the requested minimum capacity (' + minCapacity + ')');
                }
                this._resize(newCapacity);
            }
        };

        DynamicTypedArray.prototype._resize = function _resize(capacity) {
            if (capacity < 0) {
                throw new RangeError('The requested capacity must not be negative');
            }
            var byteLength = this._constructor.BYTES_PER_ELEMENT * capacity;
            var buffer = this._array === undefined ? this._allocator(byteLength) : this._reallocator(this._array.buffer, byteLength, this._allocator, this._deallocator);
            if (!(buffer instanceof ArrayBuffer)) {
                throw new TypeError('The reallocator must return an ArrayBuffer!');
            }
            if (buffer.byteLength < byteLength) {
                throw new RangeError('The allocated buffer is too small!');
            }
            this._array = new this._constructor(buffer, 0, capacity);
            if (capacity < this._length) {
                this._length = capacity;
            }
        };

        return DynamicTypedArray;
    }();

    return DynamicTypedArray;

}));
//# sourceMappingURL=dynamic-typed-array.js.map
